using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class MaterialStoredController : Controller {
        private readonly ILogger<MaterialStoredController> _logger;
        private readonly FMSContext _context;
        private readonly MaterialStoredService _service;
        private readonly StorageHelper _storage;

        public MaterialStoredController(
            ILogger<MaterialStoredController> logger,
            FMSContext context,
            MaterialStoredService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 리스트
        /// </summary>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            _logger.LogInformation("List");
            _logger.LogInformation(req.Dump());
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile file) {
            IList<Hashtable> list = new List<Hashtable>();
            using (ExcelPackage excelPackage = new ExcelPackage(file.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                for (int i = worksheet.Dimension.Start.Row; i <= worksheet.Dimension.End.Row; i++) {

                    var name = worksheet.Cells[i, 1].Value;
                    var rfid = worksheet.Cells[i, 2].Value;
                    var code = worksheet.Cells[i, 3].Value;
                    list.Add(new Hashtable {
                        {"Name", name}
                    });
                }
            }
            //_service.ImportExcel(list);
            //return BadRequest(ex.Message);

            return Ok();
        }

        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("자재 입고 목록을 찾을 수 없습니다.");
            }
            var item = data.Select(x => new {
                x.MaterialStoredId,
                x.SiteId,
                CmSite = new {
                    x.Site.Name,
                },
                x.BusinessFieldId,
                CmBusinessField = new {
                    x.CmBusinessField.Name,
                },
                x.StoredDate,
                x.PurchaseOrderId,
                FmsMaterialPurchaseOrder = new {
                    x.FmsMaterialPurchaseOrder.Title,
                    x.FmsMaterialPurchaseOrder.Reason,
                },
                x.WarehouseId,
                FmsMaterialWarehouse = new {
                    x.FmsMaterialWarehouse.Name,
                },
                x.Note,
                x.StoredUserId,
                StoredUserName = x.S.Name,
                x.ApprovalDate,
                x.ApprovalUserId,
                ApprovalUserName = x.CmUser.Name,
                x.IsApproval,
                x.LocationInWarehouse,
                x.MaterialId,
                x.StoredCount,
                x.UnitCost,
                x.RemainStoredCount,
                FmsMaterials = _context.FmsMaterialPurchaseOrderMaterial.Where(f => f.SiteId == x.SiteId && f.PurchaseOrderId == x.PurchaseOrderId).Select(f => new {
                    f.SiteId,
                    f.PurchaseOrderId,
                    SelectedMaterialId = x.MaterialId,      //입고등록 한 자재
                    f.MaterialId,
                    UnitCost = f.FmsMaterial.FinalPrice,
                    MaterialName = f.FmsMaterial.Name,
                    MaterialStandard = f.FmsMaterial.Standard,
                    MaterialUnit = f.FmsMaterial.Unit,
                    f.FmsMaterial.FirstClassId,
                    FirstClassName = f.FmsMaterial.FmsMaterialCodeClass.Name,
                    f.FmsMaterial.SecondClassId,
                    SecondClassName = f.FmsMaterial.FmsMaterialCodeClass1.Name,
                    f.FmsMaterial.ThirdClassId,
                    ThirdClassName = f.FmsMaterial.FmsMaterialCodeClass2.Name,
                    f.MaterialCount,
                    PossibleStoredCount = f.MaterialCount - Util.ToInt(_context.FmsMaterialStored.Where(fs => fs.SiteId == f.SiteId && fs.PurchaseOrderId == f.PurchaseOrderId && fs.MaterialId == f.MaterialId).Sum(fs => fs.StoredCount)),      //입고가능수량
                    StoredCount2 = _context.FmsMaterialStored.Where(ffs => ffs.SiteId == f.SiteId && ffs.PurchaseOrderId == f.PurchaseOrderId && ffs.MaterialStoredId == x.MaterialStoredId && ffs.MaterialId == f.MaterialId).Select(ffs => ffs.StoredCount).FirstOrDefault(),     //입고수량
                    StoredCount = f.MaterialCount - Util.ToInt(_context.FmsMaterialStored.Where(fms => fms.SiteId == f.SiteId && fms.PurchaseOrderId == f.PurchaseOrderId && fms.MaterialId == f.MaterialId).Sum(fms => fms.StoredCount)),      //입고 등록시 입고수량
                }).ToList(),
            });

            return Ok(item.First());
        }

        /// <summary>
        /// 등록
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        [HttpPut]
        public IActionResult Create([FromBody] FmsMaterialStored data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        /// <param name="siteId"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);

            return Ok();
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsMaterialStored> query) {
            var list = query.Select(x => new {
                x.MaterialStoredId,
                x.SiteId,
                SiteName = x.Site.Name,
                // x.BusinessFieldId,
                // BusinessFieldName = x.CmBusinessField.Name,
                // 2020.07.17 리스트에서만 재고의 업무분야 보여달라고 요청
                BusinessFieldId = x.FmsMaterial.BusinessFieldId,
                BusinessFieldName = x.FmsMaterial.CmBusinessField.Name,
                FirstClassName = x.FmsMaterial.FmsMaterialCodeClass.Name,
                SecondClassName = x.FmsMaterial.FmsMaterialCodeClass1.Name,
                ThirdClassName = x.FmsMaterial.FmsMaterialCodeClass2.Name,
                MaterialName = x.FmsMaterial.Name,
                MaterialStandard = x.FmsMaterial.Standard,
                MaterialUnit = x.FmsMaterial.Unit,
                x.StoredCount,
                ApprovalDate = (x.ApprovalDate != null) ? x.ApprovalDate.Value.ToString("yyyy-MM-dd") : "",
                x.ApprovalUserId,
                ApprovalUserName = x.CmUser.Name,
                StoredDate = x.StoredDate.ToString("yyyy-MM-dd"),
                x.PurchaseOrderId,
                x.WarehouseId,
                x.MaterialId,
                x.RemainStoredCount,
                x.UnitCost,
                x.StockCount,
                x.Note,
                x.StoredUserId,
                StoreUserName = x.S.Name,
                x.IsApproval,
                x.LocationInWarehouse,
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsMaterialStored> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 업무분야 필드가 있는경우만 추가
            if (req.businessFieldId != null) {
                query = query.Where(x => x.SiteId == req.siteId);
                query = query.Where(x => x.BusinessFieldId == req.businessFieldId || x.CmBusinessField.Name == "공통");
            }

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.Site.SortOrderNo);

            // 추가 정렬
            if (req.sort.field == "StoredAndAproval") { // 기본 커스텀 정렬 예제 (입고역순,승인일순)
                sortQuery = sortQuery.ThenByDescending(x => x.StoredDate);
                sortQuery = sortQuery.ThenBy(x => x.ApprovalDate);
            } else {
                sortQuery = sortQuery.ThenSort(req.sort);
            }

            return sortQuery;
        }

        /// <summary>
        /// 승인
        /// </summary>
        [HttpPost("{userId}")]
        public IActionResult Approval(string userId, [FromBody] ICollection<FmsMaterialStored> data) {
            if (ModelState.IsValid) {
                //if (data.Count > 0) {
                foreach (var item in data) {
                    if (item.IsApproval == false) {
                        _service.Approval(userId, item);
                    }
                }
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 개별 승인
        /// </summary>
        [HttpGet("[action]")]
        public IActionResult ApprovalEach([FromQuery] int siteId, [FromQuery] int id) {
            _service.ApprovalEach(siteId, id, User.Identity.Name);

            return Ok();
        }
    }
}